import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React, { useEffect, useReducer } from "react";
import makeImmutable, { without } from "seamless-immutable";
import MainLayout from "../MainLayout";
import SettingsProvider from "../SettingsProvider";
import combineReducers from "./reducers/combine-reducers.js";
import generalReducer from "./reducers/general-reducer.js";
import getFromLocalStorage from "../utils/get-from-local-storage";
import historyHandler from "./reducers/history-handler.js";
import imageReducer from "./reducers/image-reducer.js";
import useEventCallback from "use-event-callback";
import videoReducer from "./reducers/video-reducer.js";

function extractAllowedGroups(images) {
  var allowedGroups = [];
  images.forEach(function (image) {
    return image.regions.forEach(function (_ref) {
      var groupId = _ref.groupId;

      if (!allowedGroups.includes(groupId)) {
        allowedGroups.push(groupId);
      }
    });
  });
  return allowedGroups;
}

export var Annotator = function Annotator(_ref2) {
  var images = _ref2.images,
      allowedArea = _ref2.allowedArea,
      _ref2$selectedImage = _ref2.selectedImage,
      selectedImage = _ref2$selectedImage === void 0 ? images && images.length > 0 ? 0 : undefined : _ref2$selectedImage,
      showPointDistances = _ref2.showPointDistances,
      pointDistancePrecision = _ref2.pointDistancePrecision,
      _ref2$showTags = _ref2.showTags,
      showTags = _ref2$showTags === void 0 ? getFromLocalStorage("showTags", true) : _ref2$showTags,
      _ref2$enabledTools = _ref2.enabledTools,
      enabledTools = _ref2$enabledTools === void 0 ? ["select", "create-point", "create-box", "create-polygon", "create-line", "create-expanding-line", "show-mask"] : _ref2$enabledTools,
      _ref2$selectedTool = _ref2.selectedTool,
      selectedTool = _ref2$selectedTool === void 0 ? "select" : _ref2$selectedTool,
      _ref2$regionTagList = _ref2.regionTagList,
      regionTagList = _ref2$regionTagList === void 0 ? [] : _ref2$regionTagList,
      _ref2$regionClsList = _ref2.regionClsList,
      regionClsList = _ref2$regionClsList === void 0 ? [] : _ref2$regionClsList,
      _ref2$imageTagList = _ref2.imageTagList,
      imageTagList = _ref2$imageTagList === void 0 ? [] : _ref2$imageTagList,
      _ref2$imageClsList = _ref2.imageClsList,
      imageClsList = _ref2$imageClsList === void 0 ? [] : _ref2$imageClsList,
      _ref2$keyframes = _ref2.keyframes,
      keyframes = _ref2$keyframes === void 0 ? {} : _ref2$keyframes,
      _ref2$taskDescription = _ref2.taskDescription,
      taskDescription = _ref2$taskDescription === void 0 ? "" : _ref2$taskDescription,
      _ref2$fullImageSegmen = _ref2.fullImageSegmentationMode,
      fullImageSegmentationMode = _ref2$fullImageSegmen === void 0 ? false : _ref2$fullImageSegmen,
      RegionEditLabel = _ref2.RegionEditLabel,
      videoSrc = _ref2.videoSrc,
      _ref2$videoTime = _ref2.videoTime,
      videoTime = _ref2$videoTime === void 0 ? 0 : _ref2$videoTime,
      videoName = _ref2.videoName,
      onExit = _ref2.onExit,
      onNextImage = _ref2.onNextImage,
      onPrevImage = _ref2.onPrevImage,
      keypointDefinitions = _ref2.keypointDefinitions,
      _ref2$autoSegmentatio = _ref2.autoSegmentationOptions,
      autoSegmentationOptions = _ref2$autoSegmentatio === void 0 ? {
    type: "autoseg"
  } : _ref2$autoSegmentatio,
      hideHeader = _ref2.hideHeader,
      hideHeaderText = _ref2.hideHeaderText,
      hideNext = _ref2.hideNext,
      hidePrev = _ref2.hidePrev,
      hideClone = _ref2.hideClone,
      hideSettings = _ref2.hideSettings,
      hideFullScreen = _ref2.hideFullScreen,
      hideSave = _ref2.hideSave,
      allowComments = _ref2.allowComments,
      onImagesChange = _ref2.onImagesChange,
      groups = _ref2.groups,
      onGroupSelect = _ref2.onGroupSelect,
      hideHistory = _ref2.hideHistory,
      hideNotEditingLabel = _ref2.hideNotEditingLabel,
      showEditor = _ref2.showEditor,
      showPageSelector = _ref2.showPageSelector,
      clsColors = _ref2.clsColors,
      groupColors = _ref2.groupColors,
      onRecalc = _ref2.onRecalc,
      onSave = _ref2.onSave,
      metadata = _ref2.metadata;

  if (typeof selectedImage === "string") {
    selectedImage = (images || []).findIndex(function (img) {
      return img.src === selectedImage;
    });
    if (selectedImage === -1) selectedImage = undefined;
  }

  var allowedGroups = extractAllowedGroups(images);
  var annotationType = images ? "image" : "video";

  var _useReducer = useReducer(historyHandler(combineReducers(annotationType === "image" ? imageReducer : videoReducer, generalReducer)), makeImmutable(_objectSpread({
    annotationType: annotationType,
    showTags: showTags,
    allowedArea: allowedArea,
    showPointDistances: showPointDistances,
    pointDistancePrecision: pointDistancePrecision,
    selectedTool: selectedTool,
    fullImageSegmentationMode: fullImageSegmentationMode,
    autoSegmentationOptions: autoSegmentationOptions,
    mode: null,
    taskDescription: taskDescription,
    showMask: true,
    labelImages: imageClsList.length > 0 || imageTagList.length > 0,
    regionClsList: regionClsList,
    regionTagList: regionTagList,
    imageClsList: imageClsList,
    imageTagList: imageTagList,
    currentVideoTime: videoTime,
    enabledTools: enabledTools,
    history: [],
    videoName: videoName,
    keypointDefinitions: keypointDefinitions,
    allowComments: allowComments
  }, annotationType === "image" ? {
    selectedImage: selectedImage,
    images: images,
    selectedImageFrameTime: images && images.length > 0 ? images[0].frameTime : undefined
  } : {
    videoSrc: videoSrc,
    keyframes: keyframes
  }, {
    imagesUpdatedAt: null,
    imagesSavedAt: null,
    metadata: metadata,
    allowedGroups: allowedGroups || []
  }))),
      _useReducer2 = _slicedToArray(_useReducer, 2),
      state = _useReducer2[0],
      dispatchToReducer = _useReducer2[1];

  var dispatch = useEventCallback(function (action) {
    if (action.type === "HEADER_BUTTON_CLICKED") {
      if (["Exit", "Done", "Save", "Complete"].includes(action.buttonName)) {
        return onExit(without(state, "history"));
      } else if (action.buttonName === "Next" && onNextImage) {
        return onNextImage(without(state, "history"));
      } else if (action.buttonName === "Prev" && onPrevImage) {
        return onPrevImage(without(state, "history"));
      }
    }

    dispatchToReducer(action);
  });
  var onRegionClassAdded = useEventCallback(function (cls) {
    dispatchToReducer({
      type: "ON_CLS_ADDED",
      cls: cls
    });
  });

  var handleSaveClick =
  /*#__PURE__*/
  function () {
    var _ref3 = _asyncToGenerator(
    /*#__PURE__*/
    _regeneratorRuntime.mark(function _callee(e) {
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              e.preventDefault();

              if (!onSave) {
                _context.next = 5;
                break;
              }

              _context.next = 4;
              return onSave(state.images);

            case 4:
              dispatchToReducer({
                type: "IMAGES_SAVED",
                savedAt: new Date()
              });

            case 5:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function handleSaveClick(_x) {
      return _ref3.apply(this, arguments);
    };
  }();

  var handleRecalcClick = function handleRecalcClick(e) {
    e.preventDefault();

    if (onRecalc) {
      onRecalc();
    }
  };

  var handleMetadataChange = function handleMetadataChange(_ref4) {
    var name = _ref4.name,
        value = _ref4.value,
        imageIndex = _ref4.imageIndex;
    dispatchToReducer({
      type: "UPDATE_METADATA",
      name: name,
      value: value,
      imageIndex: imageIndex
    });
  };

  var handleAddGroup = function handleAddGroup(group) {
    dispatchToReducer({
      type: "ADD_GROUP",
      group: group
    });
  }; // trigger this on every BBox manipulation (there is currently no way to detect adding of new box!)


  useEffect(function () {
    if (!state.lastAction || !["BEGIN_BOX_TRANSFORM", "CHANGE_REGION", "DELETE_REGION"].includes(state.lastAction.type)) {
      return;
    }

    if (onImagesChange) {
      onImagesChange(state.images);
    }

    dispatchToReducer({
      type: "IMAGES_UPDATED",
      updatedAt: new Date()
    });
  }, [onImagesChange, state.images, state.lastAction]);
  useEffect(function () {
    if (selectedImage === undefined) return;
    dispatchToReducer({
      type: "SELECT_IMAGE",
      imageIndex: selectedImage,
      image: state.images[selectedImage]
    }); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [onImagesChange, selectedImage]);
  if (!images && !videoSrc) return 'Missing required prop "images" or "videoSrc"';

  var _ref5 = state.imagesSavedAt < state.imagesUpdatedAt ? [true, true] : [false, false],
      _ref6 = _slicedToArray(_ref5, 2),
      recalcActive = _ref6[0],
      saveActive = _ref6[1];

  return React.createElement(SettingsProvider, {
    clsColors: clsColors,
    groupColors: groupColors
  }, React.createElement(MainLayout, {
    RegionEditLabel: RegionEditLabel,
    alwaysShowNextButton: Boolean(onNextImage),
    alwaysShowPrevButton: Boolean(onPrevImage),
    state: state,
    dispatch: dispatch,
    onRegionClassAdded: onRegionClassAdded,
    hideHeader: hideHeader,
    hideHeaderText: hideHeaderText,
    hideNext: hideNext,
    hidePrev: hidePrev,
    hideClone: hideClone,
    hideSettings: hideSettings,
    hideFullScreen: hideFullScreen,
    hideSave: hideSave,
    groups: groups,
    onGroupSelect: onGroupSelect,
    hideHistory: hideHistory,
    hideNotEditingLabel: hideNotEditingLabel,
    showEditor: showEditor,
    showPageSelector: showPageSelector,
    onRecalc: handleRecalcClick,
    onSave: handleSaveClick,
    saveActive: recalcActive,
    recalcActive: saveActive,
    onMetadataChange: handleMetadataChange,
    onAddGroup: handleAddGroup
  }));
};
export default Annotator;